/**
   Scroll support by Javascript.
   Plural scrollers area available using this class.

   This class does not support wheel action.
   If you would like to make scrollers to respond to wheel, use
   window.addEventListener( 'wheel', 'scroller.tickUp()' );
   or others.
 */
function Scroller( containerElement, targetElement, barElement, thumbElement ) {
    this.active = false;
    if ( document.scrollers ) {
        document.scrollers.push( this );
    } else {
        document.scrollers = [ this, ];
    }

    this.container = containerElement;
    this.thumb  = thumbElement;
    this.bar    = barElement;
    this.target = targetElement;
    this.locked = false;
    this.currentPosition = 0;
    //if ( this.bar.style.appleDashboardRegion == null ) {
    this.bar.style.appleDashboardRegion = "dashboard-region(control rectangle)";
//}

    this.isActive  = function() { return this.active; };
    this.isLocked  = function() { return this.locked; };
    this.getPosition = function() { return this.currentPosition; };
    this.getBounds = __Scroller__getBounds;
    this.toString  = __Scroller__toString;
    this.refresh   = __Scroller__refresh;
    this.changeView = __Scroller__changeView;
    this.setMinimumThumbSize = __Scroler__setMinimumThumbSize;
    this.thumbDown = __Scroller__thumbDown;
    this.thumbMove = __Scroller__thumbMove;
    this.thumbUp   = __Scroller__thumbUp;
    this.skipToBarPosition = __Scroller__skipToBarPosition;
    this.tickUp    = __Scroller__tickUp;
    this.tickDown  = __Scroller__tickDown;

    this.container.scroller = this;
    this.thumb.scroller  = this;
    this.bar.scroller    = this;
    this.target.scroller = this;

    this.thumb.addEventListener( "mousedown", _Scroller_thumbDown, true );
    this.bar.addEventListener( "mouseup", _Scroller_skipToBarPosition, true );

    this.minimumThumbSize = 24;
    this.tickSteps = 16;

    this.thumbRange = { top:0, bottom:0 };
    this.thumbPosition = { start:0, current:0 };

    this.containerBounds = null;
    this.targetBounds    = null;
    this.barBounds       = null;
    this.thumbBounds     = null;

    this.visible = false;

    this.refresh();
}

// scan active scroller
function _Scroller_getActiveScroller( event ) {
    // return scroller instance if the event source contains scroller
    // field
    var elem = event.srcElement;
    if ( elem && elem.scroller ) {
        return elem.scroller;
    }

    // if the source element is direct child of the element having
    // scroller field, select the element and return the value
    if ( elem.parentNode && elem.parentNode.nodeType == 1 && elem.parentNode.scroller ) {
        return elem.parentNode.scroller;
    }

    //
    var scrollers = document.scrollers;
    if ( scrollers ) {
        for ( var i = 0; i < scrollers.length; i++ ) {
            if ( scrollers[ i ].isActive() ) {
                return scrollers[ i ];
            }
        }
    }
    debug( "no valid element" );
    return null;
}

// three following functions are called from global scope 
// they are delegations of class methods
// the thumb is captured
function _Scroller_thumbDown( event ) {
    var obj = _Scroller_getActiveScroller( event );
    if ( obj ) {
        if ( obj.isLocked() == false ) {
            obj.active = true;
            obj.thumbDown( event.y );
        }
        event.stopPropagation();
        event.preventDefault();
    }
}
// move scroll thumb and follow the action
function _Scroller_thumbMove( event ) {
    var obj = _Scroller_getActiveScroller( event );
    if ( obj ) {
        obj.thumbMove( event.y );
     event.stopPropagation();
     event.preventDefault();
    }
}
// determine the position of thumb and view
function _Scroller_thumbUp( event ) {
    var obj = _Scroller_getActiveScroller( event );
    if ( obj ) {
        obj.thumbUp( event.y );
        obj.active = false;
    }
    event.stopPropagation();
    event.preventDefault();
}

// the following methods are called from proceeding functions
function __Scroller__thumbDown( y ) {
    document.addEventListener( "mousemove", _Scroller_thumbMove, true );
    document.addEventListener( "mouseup", _Scroller_thumbUp, true );
    this.thumbPosition.start = y;
}

function __Scroller__thumbMove( y ) {
    var dy = y - this.thumbPosition.start;
    var ty = this.thumbBounds.y + dy;
    if ( ty < this.thumbRange.top ) {
        ty = this.thumbRange.top;
    }
    if ( ty > this.thumbRange.bottom ) {
        ty = this.thumbRange.bottom;
    }
    this.thumb.style.top = ty + "px";

    var pos = ( ty - this.thumbRange.top ) / ( this.thumbRange.bottom - this.thumbRange.top ) * ( this.targetBounds.height - this.containerBounds.height );
    this.changeView( pos );
    
}
function __Scroller__thumbUp( y ) {
    var dy = y - this.thumbPosition.start;
    var ty = this.thumbBounds.y + dy;
    if ( ty < this.thumbRange.top ) {
        ty = this.thumbRange.top;
    }
    if ( ty > this.thumbRange.bottom ) {
        ty = this.thumbRange.bottom;
    }
    this.thumb.style.top = ty + "px";
    this.thumbBounds.y = ty;
    //debug( "moving to " + ty + " " + dy + " , " + this.thumbBounds.y + ", " + this.thumbPosition.start + ", " + y );

    document.removeEventListener( "mousemove", _Scroller_thumbMove, true );
    document.removeEventListener( "mouseup", _Scroller_thumbUp, true );
    this.thumbPosition.current = y;

    var pos = ( ty - this.thumbRange.top ) / ( this.thumbRange.bottom - this.thumbRange.top ) * ( this.targetBounds.height - this.containerBounds.height );
    this.changeView( pos );

}

function __Scroller__changeView( position ) {
    this.currentPosition = parseInt( position );
    this.target.style.top = ( -position) + "px";
    //debug( "move to " + position );
}

function __Scroller__skipToBarPosition( y ) {
    //debug( "move to " + y );
    if ( y >= this.thumbBounds.y && y <= this.thumbBounds.heihgt + this.thumbBounds.y ) {
        return;
    }
    if ( y < this.thumbBounds.y + this.thumbBounds.height / 2 ) {
        this.tickUp();
    } else {
        this.tickDown();
    }
}

function _Scroller_skipToBarPosition( event ) {
    var scroller = _Scroller_getActiveScroller( event );
    if ( scroller ) {
        scroller.skipToBarPosition( event.y - scroller.barBounds.y );
    } else {
        debug( "no valid element" );
        return;
    }
    event.stopPropagation();
    event.preventDefault();
}

function __Scroller__tickUp() {
    //var half = this.thumbBounds.height / 2;
    this.thumbPosition.start = this.thumbBounds.y;
    var thumby = this.thumbBounds.y - this.tickSteps;// - half;
    if ( thumby < this.thumbRange.top ) {
        thumby = this.thumbRange.top;
    }
    if ( thumby > this.thumbRange.bottom ) {
        thumby = this.thumbRange.bottom;
    }
    this.thumbUp( thumby );
}

function __Scroller__tickDown() {
    //var half = this.thumbBounds.height / 2;
    this.thumbPosition.start = this.thumbBounds.y;
    var thumby = this.thumbBounds.y + this.tickSteps;// - half;
    if ( thumby < this.thumbRange.top ) {
        thumby = this.thumbRange.top;
    }
    if ( thumby > this.thumbRange.bottom ) {
        thumby = this.thumbRange.bottom;
    }
//    this.thumbMove( thumby );
    this.thumbUp( thumby );
}


function newScroller( container, target, bar, thumb ) {
    containerElement = document.getElementById( container );
    thumbElement  = document.getElementById( thumb );
    barElement    = document.getElementById( bar );
    targetElement = document.getElementById( target );
    var scroller = new Scroller( containerElement, targetElement, barElement, thumbElement );
    var bounds = scroller.getBounds( containerElement );
    var bounds = scroller.getBounds( targetElement );
    return scroller;
}

function __Scroler__setMinimumThumbSize( size ) {
    this.minimumThumbSize = size;
    this.refresh();
}

function _getPositionRecursively( element ) {
    //debug( element.id );
    if ( element.nodeType != 1 ) {
        return { x:0, y:0 };
    }
    if ( element.tagName == "BODY" ) {
        return { x:0, y:0 };
    }
    var pos;
    var parent = element.parentNode;
    if ( parent ) {
        //debug( "test parent " + parent );
        while ( parent.nodeType != 1 ) {
            if ( parent == null || parent == undefined ) {
                return {x:0, y:0};
            }
            if ( parent.parentNode ) {
                parent = parent.parentNode;
            } else {
                return {x:0,y:0};
            }
        }
        //debug( "find parent : " + parent.id );
        pos = _getPositionRecursively( parent );
    } else {
        pos = { x:0, y:0 };
    }
    var st = document.defaultView.getComputedStyle( element );
    //debug( "POS " + pos.x + ", " + pos.y + " " + st );
    var x;
    var y;
    if ( st ) {
        try {
            x = parseInt( st.getPropertyValue( "left" ) );
            y = parseInt( st.getPropertyValue( "top" ) );
        } catch ( e ) {
//            debug( e );
            x = 0;
            y = 0;
            //throw e;
        }
        //debug( "sum position : " + element.id + " : " + x + ", " + y );
//         if ( x > 0 == false && x < 0 == false ) {
//             x = 0;
//         }
//         if ( y > 0 == false && y < 0 == false ) {
//             y = 0;
//         }
    } else {
        x = 0;
        y = 0;
    }
    pos.x += x;
    pos.y += y;
    return pos;
}

function __Scroller__getBounds( element ) {
    try {
        var st = document.defaultView.getComputedStyle( element );
        var pos = _getPositionRecursively( element );
        var left   = pos.x;
        var top    = pos.y;
        var width  = parseInt( st.getPropertyValue( "width" ) );
        var height = parseInt( st.getPropertyValue( "height" ) );
        //debug( "(x, y) = " + pos.x + ", " + pos.y + " ( w, h ) = " + width + ", " + height );
        return { x:left, y:top, width:width, height:height };
    } catch ( e ) {
        //debug( element.id + " => " + e.toString() );
        debug( e );
    }
}

function __Scroller__toString() {
}

function __Scroller__refresh() {
    this.containerBounds = this.getBounds( this.container );
    this.targetBounds    = this.getBounds( this.target );
    this.barBounds       = this.getBounds( this.bar );
    this.thumbBounds     = this.getBounds( this.thumb );

    //this.thumbBounds.x = 0;
    //this.thumbBounds.y = 0;

    //this.thumb.style.top = "0px";
    //this.target.style.top = "0px";

    if ( this.containerBounds.height >= this.targetBounds.height ) {
        //this.bar.style.display = "none";
        this.visible = false;
        this.bar.style.opacity = "0";
        return;
    } else {
        //this.bar.style.display = "block";
        this.visible = true;
        this.bar.style.opacity = "1.0";
    }

    var tratio      = this.containerBounds.height / this.targetBounds.height;
    var thumbheight = this.barBounds.height * tratio;

    if ( thumbheight < this.minimumThumbSize ) {
        thumbheight = this.minimumThumbSize;
    }

    this.thumbRange = { top:0, bottom: this.barBounds.height - thumbheight + 1 };

    if ( this.cuurentPosition > this.targetBounds.height - this.containerBounds.height ) {
        this.currentPosition = this.targetBounds.height - this.containerBoudns.height;
    }
    if ( this.currentPosition < 0 ) {
        this.currentPosition = 0;
    }

    this.target.style.top = parseInt( - this.currentPosition ) + "px";

    var _v1 = this.currentPosition;
    var _v2 = this.thumbRange.bottom - this.thumbRange.top;
    var _v3 = this.targetBounds.height - this.containerBounds.height;
    var ttop = _v1 * _v2 / _v3;
    //if ( this.thumbBounds.y + this.thumbBounds.height <= this.barBounds.height ) {
    this.thumb.style.height = parseInt( thumbheight ) + "px";
        
//}

    this.thumbBounds.y = parseInt( ttop );//+ this.barBounds.y );
    this.thumb.style.top = this.thumbBounds.y + "px";
}

